aboutsummaryrefslogtreecommitdiff
path: root/src
diff options
context:
space:
mode:
Diffstat (limited to 'src')
-rw-r--r--src/CMakeLists.txt1
-rw-r--r--src/exporter.cpp1
-rw-r--r--src/haptic.cpp167
-rw-r--r--src/settings-live.cpp14
-rw-r--r--src/snap.cpp11
5 files changed, 192 insertions, 2 deletions
diff --git a/src/CMakeLists.txt b/src/CMakeLists.txt
index 754d537..a466a48 100644
--- a/src/CMakeLists.txt
+++ b/src/CMakeLists.txt
@@ -20,6 +20,7 @@ set (SERVICE_CXX_SOURCES
exporter.cpp
formatter.cpp
formatter-desktop.cpp
+ haptic.cpp
locations.cpp
locations-settings.cpp
menu.cpp
diff --git a/src/exporter.cpp b/src/exporter.cpp
index 88aee2f..1d45705 100644
--- a/src/exporter.cpp
+++ b/src/exporter.cpp
@@ -144,6 +144,7 @@ private:
bind_uint_property(m_alarm_props, "duration", m_settings->alarm_duration);
bind_uint_property(m_alarm_props, "default-volume", m_settings->alarm_volume);
bind_string_property(m_alarm_props, "default-sound", m_settings->alarm_sound);
+ bind_string_property(m_alarm_props, "haptic-feedback", m_settings->alarm_haptic);
}
/***
diff --git a/src/haptic.cpp b/src/haptic.cpp
new file mode 100644
index 0000000..a27c334
--- /dev/null
+++ b/src/haptic.cpp
@@ -0,0 +1,167 @@
+/*
+ * Copyright 2014 Canonical Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 3, as published
+ * by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranties of
+ * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
+ * PURPOSE. See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authors:
+ * Charles Kerr <charles.kerr@canonical.com>
+ */
+
+#include <notifications/dbus-shared.h>
+#include <notifications/haptic.h>
+
+#include <gio/gio.h>
+
+#include <vector>
+
+namespace unity {
+namespace indicator {
+namespace notifications {
+
+/***
+****
+***/
+
+class Haptic::Impl
+{
+public:
+
+ Impl(const Mode& mode):
+ m_mode(mode),
+ m_cancellable(g_cancellable_new())
+ {
+ g_bus_get (G_BUS_TYPE_SESSION, m_cancellable, on_bus_ready, this);
+ }
+
+ ~Impl()
+ {
+ if (m_tag)
+ g_source_remove(m_tag);
+
+ g_cancellable_cancel (m_cancellable);
+ g_object_unref (m_cancellable);
+
+ g_clear_object (&m_bus);
+ }
+
+private:
+
+ static void on_bus_ready (GObject*, GAsyncResult* res, gpointer gself)
+ {
+ GError * error;
+ GDBusConnection * bus;
+
+ error = nullptr;
+ bus = g_bus_get_finish (res, &error);
+ if (error != nullptr)
+ {
+ if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+ g_warning ("Unable to get bus: %s", error->message);
+
+ g_error_free (error);
+ }
+ else if (bus != nullptr)
+ {
+ auto self = static_cast<Impl*>(gself);
+
+ self->m_bus = G_DBUS_CONNECTION (g_object_ref (bus));
+ self->start_vibrating();
+
+ g_object_unref (bus);
+ }
+ }
+
+ void start_vibrating()
+ {
+ g_return_if_fail (m_tag == 0);
+
+ switch (m_mode)
+ {
+ case MODE_PULSE: // the only mode currently supported... :)
+ // one second on, one second off.
+ m_vibrate_pattern_msec = std::vector<uint32_t>({1000, 1000});
+ break;
+ }
+
+ // Set up a loop so that the pattern keeps repeating.
+ // NB: VibratePattern takes a repeat arg, but we avoid it because
+ // there's no way to cancel a pattern once it's started...
+ // The phone would keep vibrating long after the alarm was dismissed!
+ // So we stick to a short pattern and handle the repeat loop manually.
+ guint interval_msec = 0;
+ for (const auto& msec : m_vibrate_pattern_msec)
+ interval_msec += msec;
+ m_tag = g_timeout_add (interval_msec, on_timeout, this);
+ on_timeout (this);
+ }
+
+ static gboolean on_timeout (gpointer gself)
+ {
+ auto self = static_cast<Impl*>(gself);
+
+ // build the pattern array of uint32s
+ GVariantBuilder builder;
+ g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
+ for (const auto& msec : self->m_vibrate_pattern_msec)
+ g_variant_builder_add_value (&builder, g_variant_new_uint32(msec));
+ auto pattern_array = g_variant_builder_end (&builder);
+
+ // build the argument list
+ g_variant_builder_init (&builder, G_VARIANT_TYPE_TUPLE);
+ g_variant_builder_add_value (&builder, pattern_array);
+ g_variant_builder_add_value (&builder, g_variant_new_uint32(1u));
+ auto args = g_variant_builder_end (&builder);
+
+ g_dbus_connection_call (self->m_bus,
+ BUS_HAPTIC_NAME,
+ BUS_HAPTIC_PATH,
+ BUS_HAPTIC_INTERFACE,
+ "VibratePattern",
+ args,
+ nullptr,
+ G_DBUS_CALL_FLAGS_NONE,
+ -1,
+ self->m_cancellable,
+ nullptr,
+ nullptr);
+
+ return G_SOURCE_CONTINUE;
+ }
+
+ const Mode m_mode;
+ GCancellable * m_cancellable = nullptr;
+ GDBusConnection * m_bus = nullptr;
+ std::vector<uint32_t> m_vibrate_pattern_msec;
+ guint m_tag = 0;
+};
+
+/***
+****
+***/
+
+Haptic::Haptic(const Mode& mode):
+ impl(new Impl (mode))
+{
+}
+
+Haptic::~Haptic()
+{
+}
+
+/***
+****
+***/
+
+} // namespace datetime
+} // namespace indicator
+} // namespace unity
diff --git a/src/settings-live.cpp b/src/settings-live.cpp
index 71bbd96..a8338ed 100644
--- a/src/settings-live.cpp
+++ b/src/settings-live.cpp
@@ -55,6 +55,7 @@ LiveSettings::LiveSettings():
update_alarm_sound();
update_alarm_volume();
update_alarm_duration();
+ update_alarm_haptic();
// now listen for clients to change the properties s.t. we can sync update GSettings
@@ -130,6 +131,10 @@ LiveSettings::LiveSettings():
alarm_duration.changed().connect([this](unsigned int value){
g_settings_set_uint(m_settings, SETTINGS_ALARM_DURATION_S, value);
});
+
+ alarm_haptic.changed().connect([this](const std::string& value){
+ g_settings_set_string(m_settings, SETTINGS_ALARM_HAPTIC_S, value.c_str());
+ });
}
/***
@@ -237,6 +242,13 @@ void LiveSettings::update_alarm_duration()
alarm_duration.set(g_settings_get_uint(m_settings, SETTINGS_ALARM_DURATION_S));
}
+void LiveSettings::update_alarm_haptic()
+{
+ auto val = g_settings_get_string(m_settings, SETTINGS_ALARM_HAPTIC_S);
+ alarm_haptic.set(val);
+ g_free(val);
+}
+
/***
****
***/
@@ -284,6 +296,8 @@ void LiveSettings::update_key(const std::string& key)
update_alarm_volume();
else if (key == SETTINGS_ALARM_DURATION_S)
update_alarm_duration();
+ else if (key == SETTINGS_ALARM_HAPTIC_S)
+ update_alarm_haptic();
}
/***
diff --git a/src/snap.cpp b/src/snap.cpp
index 1506008..0b2322a 100644
--- a/src/snap.cpp
+++ b/src/snap.cpp
@@ -20,6 +20,7 @@
#include <datetime/snap.h>
#include <notifications/awake.h>
+#include <notifications/haptic.h>
#include <notifications/sound.h>
#include <gst/gst.h>
@@ -76,6 +77,12 @@ public:
const bool loop = m_engine->supports_actions();
auto sound = std::make_shared<uin::Sound>(uri, volume, loop);
+ // create the haptic feedback...
+ const auto haptic_mode = m_settings->alarm_haptic.get();
+ std::shared_ptr<uin::Haptic> haptic;
+ if (haptic_mode == "pulse")
+ haptic = std::make_shared<uin::Haptic>(uin::Haptic::MODE_PULSE);
+
// show a notification...
const auto minutes = std::chrono::minutes(m_settings->alarm_duration.get());
const bool interactive = m_engine->supports_actions();
@@ -95,10 +102,10 @@ public:
b.add_action ("dismiss", _("Dismiss"));
}
// add the 'sound' and 'awake' objects to the capture so that
// they stay alive until the closed callback is called; i.e.,
// for the lifespan of the notficiation
- b.set_closed_callback([appointment, show, dismiss, sound, awake]
+ b.set_closed_callback([appointment, show, dismiss, sound, awake, haptic]
(const std::string& action){
if (action == "show")
show(appointment);
@@ -95,10 +102,10 @@ public:
b.add_action ("dismiss", _("Dismiss"));
}
- // add the 'sound' and 'awake' objects to the capture so that
+ // add 'sound', 'haptic', and 'awake' objects to the capture so
// they stay alive until the closed callback is called; i.e.,
// for the lifespan of the notficiation